#include <stdio.h>
#include <stdlib.h>
#include <GL/glew.h>
#include <GL/glut.h>
#include <math.h>
#include <iostream>
# include <string>
# include <vector>
#include <IL/il.h>
#include <IL/ilu.h>
#include <IL/ilut.h>
#include <cstring>
#include <string>

#include "utilities/textfile.h"
using namespace std;
#ifndef _3DS_H
#define _3DS_H

//>------ Primary Chunk, at the beginning of each file
#define PRIMARY       0x4D4D

//>------ Main Chunks
#define OBJECTINFO    0x3D3D                // This gives the version of the mesh and is found right before the material and object information
#define VERSION       0x0002                // This gives the version of the .3ds file
#define EDITKEYFRAME  0xB000                // This is the header for all of the key frame info

//>------ sub defines of OBJECTINFO
#define MATERIAL      0xAFFF                // This stored the texture info
#define OBJECT        0x4000                // This stores the faces, vertices, etc...

//>------ sub defines of MATERIAL
#define MATNAME       0xA000                // This holds the material name
#define MATDIFFUSE    0xA020                // This holds the color of the object/material
#define MATMAP        0xA200                // This is a header for a new material
#define MATMAPFILE    0xA300                // This holds the file name of the texture

#define OBJECT_MESH   0x4100                // This lets us know that we are reading a new object

//>------ sub defines of OBJECT_MESH
#define OBJECT_VERTICES     0x4110          // The objects vertices
#define OBJECT_FACES        0x4120          // The objects faces
#define OBJECT_MATERIAL     0x4130          // This is found if the object has a material, either texture map or color
#define OBJECT_UV           0x4140          // The UV texture coordinates

//////////// *** NEW *** ////////// *** NEW *** ///////////// *** NEW *** ////////////////////

// This file includes all of the model structures that are needed to load
// in a .3DS file.  If you intend to do animation you will need to add on
// to this.  These structures only support the information that is needed
// to load the objects in the scene and their associative materials.

#define MAX_TEXTURES 100                                // The maximum amount of textures to load

////class to store catmul spline points
//class CatP 
//{
//public:
//    float xu, yu, zu;
//
//	CatP();
//	CatP(float xin, float yin, float zin);
//
//};
////CatP::CatP(){};
//CatP::CatP(float xin, float yin, float zin)
//{
//	xu = xin;
//	yu = yin;
//	zu = zin;
//}
#define Mag(Normal) (sqrt(Normal.x*Normal.x + Normal.y*Normal.y + Normal.z*Normal.z))
// This is our 3D point class.  This will be used to store the vertices of our model.
class CVector3 
{
public:
	CVector3() {}

	// This is our constructor that allows us to initialize our data upon creating an instance
	CVector3(float X, float Y, float Z) 
	{ 
		x = X; y = Y; z = Z;
	}

	// This returns the cross product between 2 vectors
	static CVector3 Cross(CVector3 vVector1, CVector3 vVector2)
	{
		CVector3 vCross;                                // The vector to hold the cross product
		// Get the X value
		vCross.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y));
		// Get the Y value
		vCross.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z));
		// Get the Z value
		vCross.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x));

		return vCross;                              // Return the cross product
	}

	// This returns the normal of a vector
	static CVector3 Normalize(CVector3 vNormal)
	{
		double Magnitude;                           // This holds the magitude          

		Magnitude = Mag(vNormal);                   // Get the magnitude

		vNormal.x /= (float)Magnitude;              // Divide the vector's X by the magnitude
		vNormal.y /= (float)Magnitude;              // Divide the vector's Y by the magnitude
		vNormal.z /= (float)Magnitude;              // Divide the vector's Z by the magnitude

		return vNormal;                             // Return the normal
	}

	static float Dot(CVector3 a, CVector3 b)
	{
		return (a.x*b.x + a.y*b.y +a.z*b.z );
	}

	// Here we overload the + operator so we can add vectors together 
	CVector3 operator+(CVector3 vVector)
	{
		// Return the added vectors result.
		return CVector3(vVector.x + x, vVector.y + y, vVector.z + z);
	}

	// Here we overload the - operator so we can subtract vectors 
	CVector3 operator-(CVector3 vVector)
	{
		// Return the subtracted vectors result
		return CVector3(x - vVector.x, y - vVector.y, z - vVector.z);
	}

	// Here we overload the * operator so we can multiply by scalars
	CVector3 operator*(float num)
	{
		// Return the scaled vector
		return CVector3(x * num, y * num, z * num);
	}

	// Here we overload the / operator so we can divide by a scalar
	CVector3 operator/(float num)
	{
		// Return the scale vector
		return CVector3(x / num, y / num, z / num);
	}

	float x, y, z;		
};

inline CVector3 operator *(float t, const CVector3& v)
{
	return (CVector3(t * v.x, t * v.y, t * v.z));
}

inline CVector3 operator -(const CVector3& w, const CVector3& v)
{
	return (CVector3(w.x - v.x, w.y - v.y, w.z-v.z));
}


//CVector3::CVector3();
//CVector3::CVector3(float xin, float yin, float zin)
//{
//	x = xin;
//	y = yin;
//	z = zin;
//}

// This is our 2D point class.  This will be used to store the UV coordinates.
class CVector2 
{
public:
    float x, y;
	CVector2() {}

	// This is our constructor that allows us to initialize our data upon creating an instance
	CVector2(float X, float Y) 
	{ 
		x = X; y = Y;
	}
};

// This is our face structure.  This is is used for indexing into the vertex 
// and texture coordinate arrays.  From this information we know which vertices
// from our vertex array go to which face, along with the correct texture coordinates.
class tFace
{
public:
    int vertIndex[3];           // indicies for the verts that make up this triangle
    int coordIndex[3];          // indicies for the tex coords to texture this face
};

// This holds the information for a material.  It may be a texture map of a color.
// Some of these are not used, but I left them because you will want to eventually
// read in the UV tile ratio and the UV tile offset for some models.
class tMaterialInfo
{
public:
    char  strName[255];         // The texture name
    char  strFile[255];         // The texture file name (If this is set it's a texture map)
    BYTE  color[3];             // The color of the object (R, G, B)
    int   texureId;             // the texture ID
	int   bumpId;             // the texture ID
    float uTile;                // u tiling of texture  (Currently not used)
    float vTile;                // v tiling of texture  (Currently not used)
    float uOffset;              // u offset of texture  (Currently not used)
    float vOffset;              // v offset of texture  (Currently not used)
} ;

// This holds all the information for our model/scene. 
// You should eventually turn into a robust class that 
// has loading/drawing/querying functions like:
// LoadModel(...); DrawObject(...); DrawModel(...); DestroyModel(...);
class t3DObject 
{
public:
    int  numOfVerts;            // The number of verts in the model
    int  numOfFaces;            // The number of faces in the model
    int  numTexVertex;          // The number of texture coordinates
    int  materialID;            // The texture ID to use, which is the index into our texture array
    bool bHasTexture;           // This is TRUE if there is a texture map for this object
    char strName[255];          // The name of the object
    CVector3  *pVerts;          // The object's vertices
    CVector3  *pNormals;        // The object's normals
    CVector2  *pTexVerts;       // The texture's UV coordinates
    tFace *pFaces;              // The faces information of the object
};

// This holds our model information.  This should also turn into a robust class.
// We use STL's (Standard Template Library) vector class to ease our link list burdens. :)
class t3DModel 
{
public:
    int numOfObjects;                   // The number of objects in the model
    int numOfMaterials;                 // The number of materials for the model
    vector<tMaterialInfo> pMaterials;   // The list of material information (Textures and colors)
    vector<t3DObject> pObject;          // The object list for our model
};

// Here is our structure for our 3DS indicies (since .3DS stores 4 unsigned shorts)
class tIndices {                           
public:
    unsigned short a, b, c, bVisible;       // This will hold point1, 2, and 3 index's into the vertex array plus a visible flag
};

// This holds the chunk info
class tChunk
{
public:
    unsigned short int ID;                  // The chunk's ID       
    unsigned int length;                    // The length of the chunk
    unsigned int bytesRead;                 // The amount of bytes read within that chunk
};

//model struct containing all the necessary info for loading and rendering the models
struct modelInfo
{
	//stores the actual values
	GLfloat *points;
	GLfloat *normals;
	GLfloat *texCoords;
	GLuint *indices;
	GLfloat *tangents;

	//stores the buffer pointers
	GLuint vert_IND;
	GLuint norm_IND;
	GLuint indi_IND;
	GLuint tex_IND;
	GLuint tang_IND;
	//stores tex array number
	GLuint texID;
	GLuint bumpID;

	int totalVerts;
	int totalFaces;
	GLfloat	position[3];
	GLfloat	rotation[3];

	GLint shaderID; //stores the default to-use shader
	int levelID;
};
	//static std::vector<int> testvec;
// This class handles all of the loading code
class CLoad3DS
{
public:

	ILuint DevILID;				// index fr DevIL Textur
	GLuint OpenGLID;			// Textur ID fr OpenGL
	GLuint g_Texture[100];                     // This holds the texture info, referenced by an ID
	char *FILE_NAME;
	t3DModel g_3DModel;                                     // This holds the 3D Model info that we load in
	modelInfo tempInfo;
	int arrayIndex;
	int objectCount;

	//static vector for all our models
	//std::vector<double*> aClass::vec;
	static std::vector<modelInfo> models3D;


	int vertexCount;
	GLfloat *vertices;
	t3DModel *modelData;                                     // This holds the 3D Model info that we load in
	CVector3 position;

	GLfloat *ModelPoints;
	GLfloat *ModelNormals;
	GLfloat *ModelTexCoords;
	GLuint *ModelIndices;
	int numModelverts;
	int numModelFaces;
	int totalNumModelverts;
	int totalNumModelFaces;

	float rotX;
	float rotY;
	float rotZ;

	CLoad3DS();                             // This inits the data members
    CLoad3DS(char *fn, GLuint shader, int level);                             // This inits the data members

    // This is the function that you call to load the 3DS
    bool Import3DS(t3DModel *pModel, char *strFileName);
	void loadModel();
	void renderModel();
	void translate(float x, float y, float z);
	void rotate(float x, float y, float z);
	void CreateTexture(char *strFileName,GLuint &textureID);
	float* ComputeTangents();

private:
    // This reads in a string and saves it in the char array passed in
    int GetString(char *);

    // This reads the next chunk
    void ReadChunk(tChunk *);

    // This reads the next large chunk
    void ProcessNextChunk(t3DModel *pModel, tChunk *);

    // This reads the object chunks
    void ProcessNextObjectChunk(t3DModel *pModel, t3DObject *pObject, tChunk *);

    // This reads the material chunks
    void ProcessNextMaterialChunk(t3DModel *pModel, tChunk *);

    // This reads the RGB value for the object's color
    void ReadColorChunk(tMaterialInfo *pMaterial, tChunk *pChunk);

    // This reads the objects vertices
    void ReadVertices(t3DObject *pObject, tChunk *);

    // This reads the objects face information
    void ReadVertexIndices(t3DObject *pObject, tChunk *);

    // This reads the texture coodinates of the object
    void ReadUVCoordinates(t3DObject *pObject, tChunk *);

    // This reads in the material name assigned to the object and sets the materialID
    void ReadObjectMaterial(t3DModel *pModel, t3DObject *pObject, tChunk *pPreviousChunk);
    
    // This computes the vertex normals for the object (used for lighting)
    void ComputeNormals(t3DModel *pModel);

    // This frees memory and closes the file
    void CleanUp();
    
    // The file pointer
    FILE *m_FilePointer;
    
    // These are used through the loading process to hold the chunk information
    tChunk *m_CurrentChunk;
    tChunk *m_TempChunk;
};


#endif
