#include "Camera.h"


float Magnitude(CVector3 vNormal)
{
	return (float)sqrt( (vNormal.x * vNormal.x) + 
		(vNormal.y * vNormal.y) + 
		(vNormal.z * vNormal.z) );
}


CCamera::CCamera()
{
	CVector3 vZero = CVector3(0.0, 0.0, 0.0);		// Init a vVector to 0 0 0 for our position
	CVector3 vView = CVector3(0.0, 1.0, 0.5);		// Init a starting view vVector (looking up and out the screen) 
	CVector3 vUp   = CVector3(0.0, 0.0, 1.0);		// Init a standard up vVector (Rarely ever changes)

	m_vPosition	= vZero;					// Init the position to zero
	m_vView		= vView;					// Init the view to a std starting view
	m_vUpVector	= vUp;						// Init the UpVector
}


void CCamera::PositionCamera(float positionX, float positionY, float positionZ,
							 float viewX,     float viewY,     float viewZ,
							 float upVectorX, float upVectorY, float upVectorZ)
{
	CVector3 vPosition	= CVector3(positionX, positionY, positionZ);
	CVector3 vView		= CVector3(viewX, viewY, viewZ);
	CVector3 vUpVector	= CVector3(upVectorX, upVectorY, upVectorZ);

	// The code above just makes it cleaner to set the variables.
	// Otherwise we would have to set each variable x y and z.

	m_vPosition = vPosition;					// Assign the position
	m_vView     = vView;						// Assign the view
	m_vUpVector = vUpVector;					// Assign the up vector
}


void CCamera::SetViewByMouse()
{
	POINT mousePos;									// This is a window structure that holds an X and Y
	int middleX = wWidth  >> 1;				// This is a binary shift to get half the width
	int middleY = wHeight >> 1;				// This is a binary shift to get half the height
	float angleY = 0.0f;							// This is the direction for looking up or down
	float angleZ = 0.0f;							// This will be the value we need to rotate around the Y axis (Left and Right)
	static float currentRotX = 0.0f;

	// Get the mouse's current X,Y position
	GetCursorPos(&mousePos);						

	// If our cursor is still in the middle, we never moved... so don't update the screen
	if( (mousePos.x == middleX) && (mousePos.y == middleY) ) return;

	// Set the mouse position to the middle of our window
	SetCursorPos(middleX, middleY);	
	//glutWarpPointer(middleX, middleY);	

	// Get the direction the mouse moved in, but bring the number down to a reasonable amount
	angleY = (float)( (middleX - mousePos.x) ) / 1000.0f;		
	angleZ = (float)( (middleY - mousePos.y) ) / 1000.0f;

	static float lastRotX = 0.0f; 
	lastRotX = currentRotX; // We store off the currentRotX and will use it in when the angle is capped

	// Here we keep track of the current rotation (for up and down) so that
	// we can restrict the camera from doing a full 360 loop.
	currentRotX += angleZ;

	// If the current rotation (in radians) is greater than 1.0, we want to cap it.
	if(currentRotX > 2.0f)     
	{
		currentRotX = 2.0f;

		// Rotate by remaining angle if there is any
		if(lastRotX != 2.0f) 
		{
			CVector3 vAxis = CVector3::Cross(m_vView - m_vPosition, m_vUpVector);
			vAxis = CVector3::Normalize(vAxis);

			// rotate the camera by the remaining angle (1.0f - lastRotX)
			RotateView( 2.0f - lastRotX, vAxis.x, vAxis.y, vAxis.z);
		}
	}
	// Check if the rotation is below -1.0, if so we want to make sure it doesn't continue
	else if(currentRotX < -2.0f)
	{
		currentRotX = -2.0f;

		// Rotate by the remaining angle if there is any
		if(lastRotX != -2.0f)
		{
			CVector3 vAxis = CVector3::Cross(m_vView - m_vPosition, m_vUpVector);
			vAxis = CVector3::Normalize(vAxis);

			// rotate the camera by ( -1.0f - lastRotX)
			RotateView( -2.0f - lastRotX, vAxis.x, vAxis.y, vAxis.z);
		}
	}
	// Otherwise, we can rotate the view around our position
	else 
	{	
		CVector3 vAxis = CVector3::Cross(m_vView - m_vPosition, m_vUpVector);
		vAxis = CVector3::Normalize(vAxis);

		// Rotate around our perpendicular axis
		RotateView(angleZ, vAxis.x, vAxis.y, vAxis.z);
	}

	// Always rotate the camera around the y-axis
	RotateView(angleY, 0, 1, 0);
}


void CCamera::RotateView(float angle, float x, float y, float z)
{
	CVector3 vNewView;

	// Get the view vector (The direction we are facing)
	CVector3 vView = m_vView - m_vPosition;		

	// Calculate the sine and cosine of the angle once
	float cosTheta = (float)cos(angle);
	float sinTheta = (float)sin(angle);

	// Find the new x position for the new rotated point
	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)		* vView.x;
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vView.y;
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vView.z;

	// Find the new y position for the new rotated point
	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView.x;
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)		* vView.y;
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vView.z;

	// Find the new z position for the new rotated point
	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView.x;
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vView.y;
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)		* vView.z;

	// Now we just add the newly rotated vector to our position to set
	// our new rotated view of our camera.
	m_vView = m_vPosition + vNewView;
}


void CCamera::MoveCamera(float speed)
{
	// Get our view vector (The direciton we are facing)
	CVector3 vVector = m_vView - m_vPosition;

	m_vPosition.x += vVector.x * speed;		// Add our acceleration to our position's X
	m_vPosition.z += vVector.z * speed;		// Add our acceleration to our position's Z
	m_vView.x += vVector.x * speed;			// Add our acceleration to our view's X
	m_vView.z += vVector.z * speed;			// Add our acceleration to our view's Z
}

void CCamera::RaiseCamera(float speed)
{
	// Get our view vector (The direciton we are facing)
	CVector3 vVector = m_vView - m_vPosition;
	m_vPosition.y += 1*speed;
	m_vView.y += 1*speed;
	//m_vPosition.x += vVector.x * speed;		// Add our acceleration to our position's X
	//m_vPosition.z += vVector.z * speed;		// Add our acceleration to our position's Z
	//m_vView.x += vVector.x * speed;			// Add our acceleration to our view's X
	//m_vView.z += vVector.z * speed;			// Add our acceleration to our view's Z
}

void CCamera::SetScreenRes(int width, int height)
{
	wWidth = width;
	wHeight = height;
}
