#include <fmod.h>
#include "main.h"                                        // This includes our header file
#include "3ds.h"
#include "Camera.h"
#include "utilities/textfile.h"
#include "utilities/Matrix.h"
#include "utilities/ModelView.h"
#include "utilities/Projection.h"
#include "utilities/framerate.h"
#include "utilities/tinyxml/tinyxml.h"
#include "utilities/Timer.h"

static float viewAngle=0.0f, viewDelta;
static float moveDir = 0;
static float strafeDir = 0;
static float x=0.0f,y=0.0f,z=5.0f;						//camera position
static float lx=0.0f,ly=0.0f,lz=-1.0f;					//camera viewing direction
static int wHeight = 480;
static int wWidth = 640;
static float ratio = 1.0* wWidth / wHeight;
static float farplane = 3000;
CCamera g_Camera;
static bool fullscreen = true;

static Timer time;
static framerate fps;
vector<float> controlpoints;

//level2
static bool level2 = false;
CLoad3DS port;
CLoad3DS ufo;
CLoad3DS platform;
CLoad3DS lvl1;
static float angle = 0.0;								
static float posit = 0.0;								
static float ufoAng = 0.0;								
static float ufoPos = 0.0;
float currPoints[4][3];
float currView[4][3];
TiXmlElement* cam2Node;
int pointamount2; //holds amount of points in xml file
int pcounter2; //counter for for-loop in render()
CLoad3DS skybox2;
static float event = 47;

static float event_bloom01 = 65;
static float event_bloom02 = 167;
static float event_bloom03 = 115;

//vector<modelInfo> CLoad3DS::models3D;
CLoad3DS model3D;
CLoad3DS test;
CLoad3DS test2;
CLoad3DS skybox;
CLoad3DS feisar;

ILuint g_DevILID;				// index fr DevIL Textur

//shader variablen
bool shader;
GLuint v,f,f2,p;					//shader variablen
GLuint vBl, fBl, bloomProg;
GLuint vNm, fNm, normMapProg;
GLuint vOn, fOn, neonProg;
GLuint vFl, fFl, flatProg;

/**
 * The VBO for the vertices.
 */
GLuint g_vertices;
GLuint g_normals;
GLuint g_indices;
GLuint g_texCoords;
GLuint g_texture;

//GLuint numberIndices;
/**
 * The location of the vertex in the shader program.
 */
GLint g_vertexLocation;
GLint g_normalLocation;
GLfloat g_projection[16];
GLfloat g_modelView[16];
GLint g_projectionLocation;
GLint g_modelViewLocation;
GLint g_texCoordLocation;
GLint g_textureLocation;
GLint g_tangentLoc;
GLuint g_normalMapLocation;
GLint g_lightDirLocation;

GLint bl_vertexLocation;
GLint bl_normalLocation;
GLfloat bl_projection[16];
GLfloat bl_modelView[16];
GLint bl_projectionLocation;
GLint bl_modelViewLocation;
GLint bl_texCoordLocation;
GLint bl_textureLocation;
GLint bl_textureLocation2;
GLint bl_brightnessLoc;
GLint bl_ddLoc;
static float brightness = 1.0f;
static float dd = 0.005f;

// Matrix for the model
GLfloat	model[16];
GLfloat	pos[3];
GLfloat	rot[3];

//camera path variables
bool cameraMode;
//path creation
static bool pathStarted = false;
static bool pathFinished = false;
TiXmlDocument doc;
TiXmlDeclaration * decl;
TiXmlElement * root;
TiXmlElement * points;
TiXmlElement * point;
int pointcount;
//path recreation
int pointamount; //holds amount of points in xml file
int pcounter; //counter for for-loop in render()
static float tcount = 0.0f;
float currentPoints[4][3];
float currentView[4][3];
TiXmlElement* camNode;
float xc, yc, zc;
float vx, vy, vz;
float px, py, pz;
//TiXmlHandle hRoot(NULL);
//TiXmlDocument temp();

//FBO stuff
GLuint fbo;					// Our handle to the FBO
GLuint depthBuffer;			// Our handle to the depth render buffer
GLuint img;					// Our handle to a texture
GLuint fbo2;					// Our handle to the FBO
GLuint depthBuffer2;			// Our handle to the depth render buffer
GLuint img2;					// Our handle to a texture
static float scale = 1.55f;
bool bloom;
bool testmode;
float lightpath;

FSOUND_STREAM* soundHandle;
static float timer = 0.0f;

GLuint CreateTexture(char *strFileName)
{//unsigned int textureArray[],char *strFileName,
	
	GLuint texy;
	ILuint devid;
	ilGenImages(1,&devid);			// generieren von IL ID fr Texturen
	ilBindImage(devid);			// bestimmten Texturindex als aktuell setzen

	if (!ilLoadImage ((wchar_t *) strFileName/*.c_str()*/))		// laden der Textur
	{
		ilDeleteImages(1,&devid);	// bei Fehlschlag wieder Index freigeben
		printf("\n%s image not found\n", strFileName);
		return 0;
	}
	else
	{								
		long h, v, bpp, f;
		unsigned char *texdata=0;

		h=ilGetInteger(IL_IMAGE_WIDTH);	// Breite des Bildes holen
		v=ilGetInteger(IL_IMAGE_HEIGHT);	// Hehe des Bildes holen
		bpp=ilGetInteger(IL_IMAGE_BYTES_PER_PIXEL); // Farbtiefe des Bildes
		f=ilGetInteger(IL_IMAGE_FORMAT);	// Format des Bildes z.B. RGB RGBA BGR BGRA usw.
		texdata=ilGetData();				// Zeiger auf Bilddaten holen

		//glGenTextures(1, &textureArray[textureID]);		// das gleiche gilt fr OpenGL
		//glBindTexture(GL_TEXTURE_2D, textureArray[textureID]);
		glGenTextures(1, &texy);		// das gleiche gilt fr OpenGL
		glBindTexture(GL_TEXTURE_2D, texy);

		//glGenTextures(1,&OpenGLID);		// das gleiche gilt fr OpenGL
		//glBindTexture(GL_TEXTURE_2D, OpenGLID);


		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR); // gibt an mit welchen Eigenschaften die Textur
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);  // in OpenGL dargestellt werden soll
		gluBuild2DMipmaps(GL_TEXTURE_2D, bpp, h, v, f, GL_UNSIGNED_BYTE, texdata); // erzeuge eine Mipmap Textur
		return texy;
	}
}


void setShaders() {
	//arrays for VBO quad
	//GLfloat points[] = { 
	//	0.0f, 0.0f, +0.0f, 1.0f,
	//	0.0f, +0.48f/**scale*/, +0.0f, 1.0f,
	//	+0.64f/**scale*/, +0.48f/**scale*/, +0.0f, 1.0f,
	//	+0.64f/**scale*/, 0.0f, +0.0f, 1.0f
	//};
	//GLfloat points[] = { 
	//	0.0f, 0.0f, +0.0f, 1.0f,
	//	0.0f, +0.48f*scale, +0.0f, 1.0f,
	//	+0.64f*scale, +0.48f*scale, +0.0f, 1.0f,
	//	+0.64f*scale, 0.0f, +0.0f, 1.0f
	//};
	printf("\nw: %d - h: %d", wWidth, wHeight);
	GLfloat points[] = { 
		0.0f, 0.0f, 0.0f, 1.0f,
		0.0f, wHeight, 0.0f, 1.0f,
		wWidth, wHeight, 0.0f, 1.0f,
		wWidth, 0.0f, 0.0f, 1.0f
	};
	GLfloat normals[] = { 
		+0.0f, +0.0f, +1.0f,
		+0.0f, +0.0f, +1.0f,
		+0.0f, +0.0f, +1.0f,
		+0.0f, +0.0f, +1.0f
	};
	GLuint indices[] = {	
	0,2,1,
	0,3,2};
	GLfloat cubeTexCoords[] =	{
		0.0f, 1.0f,
		0.0f, 0.0f,
		1.0f, 0.0f,
		1.0f, 1.0f
	};

	//GLfloat cubeTexCoords[] =	{
	//	0.0f, 0.0f,
	//	0.0f, 1.0f,
	//	1.0f, 1.0f,
	//	1.0f, 0.0f
	//};
	//
	shader = true;
	char *vs = NULL,*fs = NULL,*fs2 = NULL;
	char *vsBl = NULL, * fsBl = NULL;

	v = glCreateShader(GL_VERTEX_SHADER);
	f = glCreateShader(GL_FRAGMENT_SHADER);
	vBl = glCreateShader(GL_VERTEX_SHADER);
	fBl = glCreateShader(GL_FRAGMENT_SHADER);


	vs = textFileRead("shader/Vertex.vs");
	fs = textFileRead("shader/Fragment.fs");
	//vs = textFileRead("normalmapping.vs");
	//fs = textFileRead("normalmapping.fs");
	vsBl = textFileRead("shader/bloom.vs");
	fsBl = textFileRead("shader/bloom.fs");
	

	const char * ff = fs;
	const char * vv = vs;
	const char * ffBl = fsBl;
	const char * vvBl = vsBl;


	glShaderSource(v, 1, &vv,NULL);
	glShaderSource(f, 1, &ff,NULL);
	glShaderSource(vBl, 1, &vvBl,NULL);
	glShaderSource(fBl, 1, &ffBl,NULL);

	free(vs);free(fs);
	free(vsBl);free(fsBl);

	glCompileShader(v);
	glCompileShader(f);
	glCompileShader(vBl);
	glCompileShader(fBl);

	p = glCreateProgram();
	glAttachShader(p,f);
	glAttachShader(p,v);
	glLinkProgram(p);

	bloomProg = glCreateProgram();
	glAttachShader(bloomProg,fBl);
	glAttachShader(bloomProg,vBl);
	glLinkProgram(bloomProg);

	char *vsNm = NULL, * fsNm = NULL;
	vNm = glCreateShader(GL_VERTEX_SHADER);
	fNm = glCreateShader(GL_FRAGMENT_SHADER);
	vsNm = textFileRead("shader/normalmapping.vs");
	fsNm = textFileRead("shader/normalmapping.fs");
	const char * ffNm = fsNm;
	const char * vvNm = vsNm;
	glShaderSource(vNm, 1, &vvNm,NULL);
	glShaderSource(fNm, 1, &ffNm,NULL);
	free(vsNm);free(fsNm);
	glCompileShader(vNm);
	glCompileShader(fNm);
	normMapProg = glCreateProgram();
	glAttachShader(normMapProg,fNm);
	glAttachShader(normMapProg,vNm);
	glLinkProgram(normMapProg);

	char *vsOn = NULL, * fsOn = NULL;
	vOn = glCreateShader(GL_VERTEX_SHADER);
	fOn = glCreateShader(GL_FRAGMENT_SHADER);
	vsOn = textFileRead("shader/neon.vs");
	fsOn = textFileRead("shader/neon.fs");
	const char * ffOn = fsOn;
	const char * vvOn = vsOn;
	glShaderSource(vOn, 1, &vvOn,NULL);
	glShaderSource(fOn, 1, &ffOn,NULL);
	free(vsOn);free(fsOn);
	glCompileShader(vOn);
	glCompileShader(fOn);
	neonProg = glCreateProgram();
	glAttachShader(neonProg,fOn);
	glAttachShader(neonProg,vOn);
	glLinkProgram(neonProg);

	char *vsFl = NULL, * fsFl = NULL;
	vFl = glCreateShader(GL_VERTEX_SHADER);
	fFl = glCreateShader(GL_FRAGMENT_SHADER);
	vsFl = textFileRead("shader/flat.vs");
	fsFl = textFileRead("shader/flat.fs");
	const char * ffFl = fsFl;
	const char * vvFl = vsFl;
	glShaderSource(vFl, 1, &vvFl,NULL);
	glShaderSource(fFl, 1, &ffFl,NULL);
	free(vsFl);free(fsFl);
	glCompileShader(vFl);
	glCompileShader(fFl);
	flatProg = glCreateProgram();
	glAttachShader(flatProg,fFl);
	glAttachShader(flatProg,vFl);
	glLinkProgram(flatProg);

	//shader handles fr beleuchtung der szene
	g_projectionLocation = glGetUniformLocation(p, "projectionMatrix");
	g_modelViewLocation = glGetUniformLocation(p, "modelViewMatrix");
	g_normalLocation = glGetAttribLocation(p, "normal");
	g_vertexLocation = glGetAttribLocation(p, "vertex");
	glBindFragDataLocation(p, 0, "fragColor");
	g_textureLocation = glGetUniformLocation(p, "firstTexture");
	g_texCoordLocation = glGetAttribLocation(p, "texCoord");
	g_tangentLoc = glGetAttribLocation(p, "tangent");
	g_normalMapLocation = glGetUniformLocation(p, "normalMap");
	g_lightDirLocation = glGetUniformLocation(p, "lightDir");

	//shader handles fr FBO
	bl_projectionLocation = glGetUniformLocation(bloomProg, "projectionMatrix");
	bl_modelViewLocation = glGetUniformLocation(bloomProg, "modelViewMatrix");
	bl_normalLocation = glGetAttribLocation(bloomProg, "normal");
	bl_vertexLocation = glGetAttribLocation(bloomProg, "vertex");
	glBindFragDataLocation(bloomProg, 0, "fragColor");
	bl_textureLocation = glGetUniformLocation(bloomProg, "firstTexture");
	bl_textureLocation2 = glGetUniformLocation(bloomProg, "secondTexture");
	bl_texCoordLocation = glGetAttribLocation(bloomProg, "texCoord");
	bl_brightnessLoc = glGetUniformLocation(bloomProg, "brightness");
	bl_ddLoc = glGetUniformLocation(bloomProg, "dd");

	// alle models durchgehen, buffer generieren, buffer fttern, 
	for(int i=0; i<CLoad3DS::models3D.size(); i++)
	{
		//printf("\nV buffer %d: %d",i,sizeof(CLoad3DS::models3D.at(i).points));
		//printf("\ntotal verts %d: %d\n",i,CLoad3DS::models3D.at(i).totalVerts);
		glGenBuffers(1, &CLoad3DS::models3D.at(i).vert_IND);
		//printf("\nVertBuffer#: %d\n",CLoad3DS::models3D.at(i).vert_IND);
		glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).vert_IND);
		//3*4*sizeof_float weil nur 3 punkte, mit jeweils 4 floatwerten i.d. buffer geschrieben werden
		//DYNAMIC DRAW verwenden wenn die vbos oft modifiziert werden
		glBufferData(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).totalVerts*4*sizeof(GLfloat), (GLfloat*)CLoad3DS::models3D.at(i).points, GL_STATIC_DRAW);
		glVertexAttribPointer(g_vertexLocation, 4, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(g_vertexLocation);

		glGenBuffers(1, &CLoad3DS::models3D.at(i).norm_IND);
		//printf("\nNormBuffer#: %d\n",CLoad3DS::models3D.at(i).norm_IND);
		glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).norm_IND);
		glBufferData(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).totalVerts*3*sizeof(GLfloat), (GLfloat*)CLoad3DS::models3D.at(i).normals, GL_STATIC_DRAW);
		glVertexAttribPointer(g_normalLocation, 3, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(g_normalLocation);

		glGenBuffers(1, &CLoad3DS::models3D.at(i).indi_IND);
		//printf("\nIndBuffer#: %d\n",CLoad3DS::models3D.at(i).indi_IND);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, CLoad3DS::models3D.at(i).indi_IND);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, CLoad3DS::models3D.at(i).totalFaces*3*sizeof(GLuint), (GLuint*)CLoad3DS::models3D.at(i).indices, GL_STATIC_DRAW);

		glGenBuffers(1, &CLoad3DS::models3D.at(i).tex_IND);
		//printf("\nTexBuffer#: %d\n",CLoad3DS::models3D.at(i).tex_IND);
		glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).tex_IND); //24 = numofvertices des cubes
		glBufferData(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).totalVerts*2*sizeof(GLfloat), (GLfloat*)CLoad3DS::models3D.at(i).texCoords, GL_STATIC_DRAW);
		glVertexAttribPointer(g_texCoordLocation, 2, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(g_texCoordLocation);
		//glUniform1i(g_textureLocation, 0);
		//glUniform1i(g_normalMapLocation, 1);
		//glActiveTexture(GL_TEXTURE1);
		//glBindTexture(GL_TEXTURE_2D, tg_normalMap);

		glGenBuffers(1, &CLoad3DS::models3D.at(i).tang_IND);
		//printf("\nNormBuffer#: %d\n",CLoad3DS::models3D.at(i).tang_IND);
		glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).tang_IND);
		glBufferData(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).totalVerts*4*sizeof(GLfloat), (GLfloat*)CLoad3DS::models3D.at(i).tangents, GL_STATIC_DRAW);
		glVertexAttribPointer(g_tangentLoc, 4, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(g_tangentLoc);
	}
	glUseProgram(p);
	glUniform1i(g_textureLocation, 0);
	glUniform1i(g_normalMapLocation, 1);

	//VBO fr FBO quad
	glGenBuffers(1, &g_vertices);
	glBindBuffer(GL_ARRAY_BUFFER, g_vertices);
	glBufferData(GL_ARRAY_BUFFER, 4*4*sizeof(GLfloat), (GLfloat*)points, GL_STATIC_DRAW);
	glVertexAttribPointer(bl_vertexLocation, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glEnableVertexAttribArray(bl_vertexLocation);

	glGenBuffers(1, &g_normals);
	glBindBuffer(GL_ARRAY_BUFFER, g_normals);
	glBufferData(GL_ARRAY_BUFFER, 4*3*sizeof(GLfloat), (GLfloat*)normals, GL_STATIC_DRAW);
	glVertexAttribPointer(bl_normalLocation, 3, GL_FLOAT, GL_FALSE, 0, 0);
	glEnableVertexAttribArray(bl_normalLocation);

	glGenBuffers(1, &g_indices);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indices);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, 2*3*sizeof(GLuint), (GLuint*)indices, GL_STATIC_DRAW);

	glGenBuffers(1, &g_texCoords);
	glBindBuffer(GL_ARRAY_BUFFER, g_texCoords);
	glBufferData(GL_ARRAY_BUFFER, 4*2*sizeof(GLfloat), (GLfloat*)cubeTexCoords, GL_STATIC_DRAW);
	glVertexAttribPointer(bl_texCoordLocation, 2, GL_FLOAT, GL_FALSE, 0, 0);
	glEnableVertexAttribArray(bl_texCoordLocation);
	glUniform1i(bl_textureLocation, 0);
}

bool initGL()
{
	ilInit();

	//glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(1.0f, 1.0f, 1.0f, 0.0f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
	//glDisable(CLIP_DISTANCE0);
	//const double plane[4] = {1000, 1000, 1000, 1000};
	//glClipPlane(0,  plane);
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	//glEnable(GL_CULL_FACE);

	//glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	//glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
	return true;										// Initialization Went OK
}

void resize(int width, int height)
{
	if(height == 0)
		height = 1;

	ratio = 1.0* width / height;

	// Set the viewport to be the entire window
	glViewport(0, 0, width, height);

	wWidth = width;
	wHeight = height;
	g_Camera.SetScreenRes(width, height);
	// Calculate the projection matrix and set it
	Projection::perspective(g_projection, 40.0f, ratio, 0.0f, farplane);
	glUniformMatrix4fv(g_projectionLocation, 1, GL_FALSE, g_projection);
	
}

void switchLevel()
{
	bloom = false;
	level2 = true;
	tcount = 0;
	brightness = 0;
	port.translate(0,0,0);
	ufo.translate(0,0,0);
	platform.translate(0,0,0);
	lvl1.translate(0,0,1000);
	skybox.translate(0,0,1000);
	FSOUND_Stream_Stop (soundHandle);
	soundHandle = FSOUND_Stream_Open("lvl2.mp3",0, 0, 0);
	FSOUND_Stream_Play(0,soundHandle);
	event = event + time.getElapsedTime();
}


void startCameraPath()
{
	pathStarted = true;
	pointcount = 0;

	decl = new TiXmlDeclaration( "1.0", "", "" );
	doc.LinkEndChild( decl );
	
	root = new TiXmlElement( "camerapath" );
	doc.LinkEndChild( root );

	points = new TiXmlElement("points");
	root->LinkEndChild( points );

	printf("\ncam path started!");
}

void addCameraNode()
{
	point = new TiXmlElement("pos"); 
	point->SetDoubleAttribute("x", g_Camera.m_vPosition.x);
	point->SetDoubleAttribute("y", g_Camera.m_vPosition.y);
	point->SetDoubleAttribute("z", g_Camera.m_vPosition.z);
	points->LinkEndChild( point );

	point = new TiXmlElement("view"); 
	point->SetDoubleAttribute("xv", g_Camera.m_vView.x);
	point->SetDoubleAttribute("yv", g_Camera.m_vView.y);
	point->SetDoubleAttribute("zv", g_Camera.m_vView.z);
	points->LinkEndChild( point );
	pointcount++;

	printf("\ncam node added!");
}

void saveCameraPath()
{
	pathFinished = true;
	
	points->SetAttribute("amount", pointcount);
	doc.SaveFile( "campath.xml" );

	printf("\ncam path finished!");
}


void processKeys(unsigned char key, int x, int y)
{
	switch(key)
	{
		case 'x':
			shader = !shader;
			if(shader)
				glUseProgram(p);
			else glUseProgram(0);
			break;
		case 'q':
			if(!pathStarted)
				startCameraPath();
			break;
		case 'e':
			if(pathStarted && !pathFinished)
				saveCameraPath();
			break;
		case 'w':
			if(pathStarted && !pathFinished)
				addCameraNode();
			break;
		case 'c':
			cameraMode = !cameraMode;
			break;
		case 'o':
			dd += 0.0005;
			printf("\ndd: %f", dd);
			break;
		case 'l':
			dd -= 0.0005;
			printf("\ndd: %f", dd);
			break;
		case 'b':
			bloom = !bloom;
			break;
		case 's':
			switchLevel();
			break;
		case 27:
			FSOUND_Stream_Stop (soundHandle);
			FSOUND_Stream_Close(soundHandle);
			FSOUND_Close ();
			exit(0);
			break;
	}
}

void moveCam(int direction)
{
	x = x + direction*(lx)*0.1;
	z = z + direction*(lz)*0.1;
}

void processSpecialKeys(int key, int x, int y)
{
	switch(key) {
		case GLUT_KEY_LEFT:
			strafeDir = 0.25;
			break;
		case GLUT_KEY_RIGHT:
			strafeDir = -0.25;
			break;
		case GLUT_KEY_UP:
			moveDir = 0.25;
			break;
		case GLUT_KEY_DOWN:
			moveDir = -0.25;
			break;
	}
}

void releaseSpecialKeys(int key, int x, int y)
{
	switch(key) {
		case GLUT_KEY_LEFT:
		case GLUT_KEY_RIGHT:
			strafeDir = 0;
		case GLUT_KEY_UP:
		case GLUT_KEY_DOWN:
			moveDir = 0;
			break;
	}
}

void passiveMouseMotion(int x, int y)
{
	viewAngle = ((float)x - 320)/100;
	//printf("\npos x: %d , pos y: %d", x, y);

    lx = sin(viewAngle);
    lz = -cos(viewAngle);
}

//void loadXML()
//{
//	cameraMode = true;
//	
//	bool loaded = temp->LoadFile("campath.xml");
//
//	if(loaded)
//	{
//		printf("\nxml file loaded.");
//	}
//	else printf("\nxml file not found.");
//	float xc, yc, zc;
//
//	TiXmlHandle hDoc(&temp);
//	TiXmlElement* pElem=hDoc.FirstChildElement().Element();
//	hRoot=TiXmlHandle(pElem);
//
//	pcounter = 0;
//	hRoot.FirstChild("points").Element()->QueryIntAttribute("amount", &pointamount);
//	printf("\nnumber of points: %d", pointamount);
//
//	camNode=hRoot.FirstChild( "points" ).FirstChild().Element();
//
//	//save the first 4 points of our camera path
//	for (int i=0; i<4; i++)
//	{
//		camNode->QueryFloatAttribute("x", &xc); // If this fails, original value is left as-is
//		camNode->QueryFloatAttribute("y", &yc);
//		camNode->QueryFloatAttribute("z", &zc);
//		printf("\npos values x='%f', y='%f', z='%f'", xc, yc, zc);
//		currentPoints[i][0] = xc;
//		currentPoints[i][1] = yc;
//		currentPoints[i][2] = zc;
//
//		camNode=camNode->NextSiblingElement();
//		camNode->QueryFloatAttribute("xv", &xc);
//		camNode->QueryFloatAttribute("yv", &yc);
//		camNode->QueryFloatAttribute("zv", &zc);
//		currentView[i][0] = xc;
//		currentView[i][1] = yc;
//		currentView[i][2] = zc;
//		printf("\nview values x='%f', y='%f', z='%f'", xc, yc, zc);
//
//		if(i!=3)
//			camNode=camNode->NextSiblingElement();
//	}
//	//printf("\nyoyoyo!!!");
//}
//
void getNextPoint()
{
	//printf("\n*******************************************");
	//the previous last 3 points become our current first 3 points
	for (int i=0; i<3; i++)
	{
		currentPoints[i][0] = currentPoints[i+1][0];
		currentPoints[i][1] = currentPoints[i+1][1];
		currentPoints[i][2] = currentPoints[i+1][2];
		//printf("\npos values x='%f', y='%f', z='%f'", currentPoints[i][0], currentPoints[i][1], currentPoints[i][2]);

		currentView[i][0] = currentView[i+1][0];
		currentView[i][1] = currentView[i+1][1];
		currentView[i][2] = currentView[i+1][2];
		//printf("\nview values x='%f', y='%f', z='%f'", currentView[i][0], currentView[i][1], currentView[i][2]);
	}
	//printf("\nyeah1111111111111!!!");
	//the current 4th point is read from the xml file
	camNode=camNode->NextSiblingElement();
	camNode->QueryFloatAttribute("x", &xc); // If this fails, original value is left as-is
	camNode->QueryFloatAttribute("y", &yc);
	camNode->QueryFloatAttribute("z", &zc);
	currentPoints[3][0] = xc;
	currentPoints[3][1] = yc;
	currentPoints[3][2] = zc;
	//printf("\npos values x='%f', y='%f', z='%f'", currentPoints[3][0], currentPoints[3][1], currentPoints[3][2]);
	//printf("\nyeah22222222222222!!!");
	camNode=camNode->NextSiblingElement();
	camNode->QueryFloatAttribute("xv", &xc);
	camNode->QueryFloatAttribute("yv", &yc);
	camNode->QueryFloatAttribute("zv", &zc);
	currentView[3][0] = xc;
	currentView[3][1] = yc;
	currentView[3][2] = zc;
	//printf("\nview values x='%f', y='%f', z='%f'", currentView[3][0], currentView[3][1], currentView[3][2]);
	//printf("\nyeah33333333333333333!!!");
}

void getNextPoint2()
{
	//printf("\n*******************************************");
	//the previous last 3 points become our current first 3 points
	for (int i=0; i<3; i++)
	{
		currPoints[i][0] = currPoints[i+1][0];
		currPoints[i][1] = currPoints[i+1][1];
		currPoints[i][2] = currPoints[i+1][2];
		//printf("\npos values x='%f', y='%f', z='%f'", currPoints[i][0], currPoints[i][1], currPoints[i][2]);

		currView[i][0] = currView[i+1][0];
		currView[i][1] = currView[i+1][1];
		currView[i][2] = currView[i+1][2];
		//printf("\nview values x='%f', y='%f', z='%f'", currView[i][0], currView[i][1], currView[i][2]);
	}
	//printf("\nyeah1111111111111!!!");
	//the curr 4th point is read from the xml file
	cam2Node=cam2Node->NextSiblingElement();
	cam2Node->QueryFloatAttribute("x", &xc); // If this fails, original value is left as-is
	cam2Node->QueryFloatAttribute("y", &yc);
	cam2Node->QueryFloatAttribute("z", &zc);
	currPoints[3][0] = xc;
	currPoints[3][1] = yc;
	currPoints[3][2] = zc;
	//printf("\npos values x='%f', y='%f', z='%f'", currPoints[3][0], currPoints[3][1], currPoints[3][2]);
	//printf("\nyeah22222222222222!!!");
	cam2Node=cam2Node->NextSiblingElement();
	cam2Node->QueryFloatAttribute("xv", &xc);
	cam2Node->QueryFloatAttribute("yv", &yc);
	cam2Node->QueryFloatAttribute("zv", &zc);
	currView[3][0] = xc;
	currView[3][1] = yc;
	currView[3][2] = zc;
	//printf("\nview values x='%f', y='%f', z='%f'", currentView[3][0], currentView[3][1], currentView[3][2]);
	//printf("\nyeah33333333333333333!!!");
}

void renderScene()
{
	//printf("\nTIME: %f", time.getElapsedTime());
	
	//printf("\ntime: %f", timer);
	//timer += 0.0001;
	//if(timer > 0.13)
	//	bloom = true;
	//if(timer > 1.5)
	//	exit(0);

	if(moveDir != 0)
		g_Camera.MoveCamera(moveDir);
	if(strafeDir != 0)
		g_Camera.RaiseCamera(strafeDir);
	if(!cameraMode)
		g_Camera.SetViewByMouse();

	fps.SetSpeedFactor();
	//printf("\nspeedfactor:%f", fps.speedfactor);

	//GLfloat lightDir[] = {2.0f*cosf(angle), 0.0f, 1.0f};58.740887" yv="23.609528" zv="38.583008
	//GLfloat lightDir[] = {58.7, 23.0, 38.5};
	GLfloat lightDir[] = {2.0, 0.0, 1.0};
	glUniform3fv(g_lightDirLocation, 1, (GLfloat*)lightDir);	

//**********************************************************************************************
//*********************LEVEL2 ROUTINE
if (level2)
{
	// First we bind the FBO so we can render to it
	if(bloom)
	{
		glBindFramebuffer(GL_FRAMEBUFFER, fbo);
		glViewport(0,0,wWidth,wHeight);
	}
	glClearColor(1.0, 1.0, 1.0, 1.0);
	//glClearColor(0.0, 0.0, 0.0, 1.0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer

	// alle models durchgehen, binden, rendern
	for(int i=0; i<CLoad3DS::models3D.size(); i++)
	{
if(CLoad3DS::models3D.at(i).levelID ==2)
{
		if (CLoad3DS::models3D.at(i).shaderID == 0)
		{
			glUseProgram(p);
			g_projectionLocation = glGetUniformLocation(p, "projectionMatrix");
			g_modelViewLocation = glGetUniformLocation(p, "modelViewMatrix");
			g_normalLocation = glGetAttribLocation(p, "normal");
			g_vertexLocation = glGetAttribLocation(p, "vertex");
			glBindFragDataLocation(p, 0, "fragColor");
			g_textureLocation = glGetUniformLocation(p, "firstTexture");
			g_texCoordLocation = glGetAttribLocation(p, "texCoord");
			g_tangentLoc = glGetAttribLocation(p, "tangent");
			g_normalMapLocation = glGetUniformLocation(p, "normalMap");
			g_lightDirLocation = glGetUniformLocation(p, "lightDir");
		}
		else if (CLoad3DS::models3D.at(i).shaderID == 1)
		{
			glUseProgram(normMapProg);
			g_projectionLocation = glGetUniformLocation(normMapProg, "projectionMatrix");
			g_modelViewLocation = glGetUniformLocation(normMapProg, "modelViewMatrix");
			g_normalLocation = glGetAttribLocation(normMapProg, "normal");
			g_vertexLocation = glGetAttribLocation(normMapProg, "vertex");
			glBindFragDataLocation(normMapProg, 0, "fragColor");
			g_textureLocation = glGetUniformLocation(normMapProg, "firstTexture");
			g_texCoordLocation = glGetAttribLocation(normMapProg, "texCoord");
			g_tangentLoc = glGetAttribLocation(normMapProg, "tangent");
			g_normalMapLocation = glGetUniformLocation(normMapProg, "normalMap");
			g_lightDirLocation = glGetUniformLocation(normMapProg, "lightDir");
		}
		else if (CLoad3DS::models3D.at(i).shaderID == 2)
		{
			glUseProgram(neonProg);
			g_projectionLocation = glGetUniformLocation(neonProg, "projectionMatrix");
			g_modelViewLocation = glGetUniformLocation(neonProg, "modelViewMatrix");
			g_normalLocation = glGetAttribLocation(neonProg, "normal");
			g_vertexLocation = glGetAttribLocation(neonProg, "vertex");
			glBindFragDataLocation(neonProg, 0, "fragColor");
			g_textureLocation = glGetUniformLocation(neonProg, "firstTexture");
			g_texCoordLocation = glGetAttribLocation(neonProg, "texCoord");
			g_tangentLoc = glGetAttribLocation(neonProg, "tangent");
			g_normalMapLocation = glGetUniformLocation(neonProg, "normalMap");
			g_lightDirLocation = glGetUniformLocation(neonProg, "lightDir");
		}
		if (CLoad3DS::models3D.at(i).shaderID == 3)
		{
			glUseProgram(flatProg);
			g_projectionLocation = glGetUniformLocation(flatProg, "projectionMatrix");
			g_modelViewLocation = glGetUniformLocation(flatProg, "modelViewMatrix");
			g_normalLocation = glGetAttribLocation(flatProg, "normal");
			g_vertexLocation = glGetAttribLocation(flatProg, "vertex");
			glBindFragDataLocation(flatProg, 0, "fragColor");
			g_textureLocation = glGetUniformLocation(flatProg, "firstTexture");
			g_texCoordLocation = glGetAttribLocation(flatProg, "texCoord");
			g_tangentLoc = glGetAttribLocation(flatProg, "tangent");
			g_normalMapLocation = glGetUniformLocation(flatProg, "normalMap");
			g_lightDirLocation = glGetUniformLocation(flatProg, "lightDir");
		}
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, /*tg_texture*/CLoad3DS::models3D.at(i).texID);

			glActiveTexture(GL_TEXTURE1);
			glBindTexture(GL_TEXTURE_2D, CLoad3DS::models3D.at(i).bumpID/*tg_normalMap*/);

			glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).vert_IND);
			glVertexAttribPointer(g_vertexLocation, 4, GL_FLOAT, GL_FALSE, 0, 0);
			glEnableVertexAttribArray(g_vertexLocation);

			glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).norm_IND);
			glVertexAttribPointer(g_normalLocation, 3, GL_FLOAT, GL_FALSE, 0, 0);
			glEnableVertexAttribArray(g_normalLocation);

			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, CLoad3DS::models3D.at(i).indi_IND);

			glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).tex_IND);
			glVertexAttribPointer(g_texCoordLocation, 2, GL_FLOAT, GL_FALSE, 0, 0);
			glEnableVertexAttribArray(g_texCoordLocation);
			glUniform1i(g_textureLocation, 0); //bergibt die texturlocation (0) an den shader
			glUniform1i(g_normalMapLocation, 1);

			glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).tang_IND);
			glVertexAttribPointer(g_tangentLoc, 4, GL_FLOAT, GL_FALSE, 0, 0);
			glEnableVertexAttribArray(g_tangentLoc);
		
		if (tcount > 1.0f)
		{
			tcount = 0.0f;
			pcounter++;
			if(pcounter<pointamount-3)
				getNextPoint();
		}

		px = 0.5 *((2 * currentPoints[1][0]) + (-currentPoints[0][0] + currentPoints[2][0]) * tcount + (2*currentPoints[0][0] - 5*currentPoints[1][0] + 4*currentPoints[2][0] - currentPoints[3][0]) * tcount*tcount + (-currentPoints[0][0] + 3*currentPoints[1][0]- 3*currentPoints[2][0] + currentPoints[3][0]) * tcount*tcount*tcount);
		py = 0.5 *((2 * currentPoints[1][1]) + (-currentPoints[0][1] + currentPoints[2][1]) * tcount + (2*currentPoints[0][1] - 5*currentPoints[1][1] + 4*currentPoints[2][1] - currentPoints[3][1]) * tcount*tcount + (-currentPoints[0][1] + 3*currentPoints[1][1]- 3*currentPoints[2][1] + currentPoints[3][1]) * tcount*tcount*tcount);
		pz = 0.5 *((2 * currentPoints[1][2]) + (-currentPoints[0][2] + currentPoints[2][2]) * tcount + (2*currentPoints[0][2] - 5*currentPoints[1][2] + 4*currentPoints[2][2] - currentPoints[3][2]) * tcount*tcount + (-currentPoints[0][2] + 3*currentPoints[1][2]- 3*currentPoints[2][2] + currentPoints[3][2]) * tcount*tcount*tcount);

		vx = 0.5 *((2 * currentView[1][0]) + (-currentView[0][0] + currentView[2][0]) * tcount + (2*currentView[0][0] - 5*currentView[1][0] + 4*currentView[2][0] - currentView[3][0]) * tcount*tcount + (-currentView[0][0] + 3*currentView[1][0]- 3*currentView[2][0] + currentView[3][0]) * tcount*tcount*tcount);
		vy = 0.5 *((2 * currentView[1][1]) + (-currentView[0][1] + currentView[2][1]) * tcount + (2*currentView[0][1] - 5*currentView[1][1] + 4*currentView[2][1] - currentView[3][1]) * tcount*tcount + (-currentView[0][1] + 3*currentView[1][1]- 3*currentView[2][1] + currentView[3][1]) * tcount*tcount*tcount);
		vz = 0.5 *((2 * currentView[1][2]) + (-currentView[0][2] + currentView[2][2]) * tcount + (2*currentView[0][2] - 5*currentView[1][2] + 4*currentView[2][2] - currentView[3][2]) * tcount*tcount + (-currentView[0][2] + 3*currentView[1][2]- 3*currentView[2][2] + currentView[3][2]) * tcount*tcount*tcount);

		for(int j=0; j<3; j++)
		{
			pos[j] = CLoad3DS::models3D.at(i).position[j];
			rot[j] = CLoad3DS::models3D.at(i).rotation[j];
		}

		//if(time.getElapsedTime() > 35.0 && time.getElapsedTime() < 47.0)
		if(time.getElapsedTime() > event && time.getElapsedTime() < event+5)
		{
			posit += 0.001*fps.speedfactor;
			platform.rotate(0,angle,0);
			ufo.rotate(0,angle,0);
			platform.translate(0,posit,0);
			ufo.translate(0,posit,0);

		}
		//if(time.getElapsedTime() > 46.0 /*&& time.getElapsedTime() < 47.0*/)
		if(time.getElapsedTime() > event+5 && time.getElapsedTime() < event+9)
		{
			ufoAng -= 0.002*fps.speedfactor;
			ufo.rotate(0,angle*10,ufoAng);
		}
		if(time.getElapsedTime() > event+9 /*&& time.getElapsedTime() < 17.0*/)
		{
			ufoPos += 0.005*fps.speedfactor;
			ufo.rotate(0,angle*30,ufoAng);
			ufo.translate(ufoPos,posit+ufoPos,0);

		}

		//Projection::orthoFrustum(g_projection, 100, 100, -1000, 400);
		Projection::perspective(g_projection, 40.0f, ratio, 1.0f, farplane);
		glUniformMatrix4fv(g_projectionLocation, 1, GL_FALSE, g_projection);

		if(cameraMode)
			skybox2.translate(px, py-80, pz);
		else skybox2.translate(g_Camera.m_vPosition.x, g_Camera.m_vPosition.y-80, g_Camera.m_vPosition.z);

		Matrix::loadIdentity(model);
		Matrix::translateMatrix(model, pos[0], pos[1], pos[2]);
		Matrix::rotateRzRyRxMatrixf(model, rot[0], rot[1], rot[2]);

		// ... and the view matrix ...
		if(cameraMode)
			ModelView::lookAt(g_modelView, px, py, pz, vx, vy, vz, 0.0f, 1.0f, 0.0f);
		else ModelView::lookAt(g_modelView, g_Camera.m_vPosition.x, g_Camera.m_vPosition.y, g_Camera.m_vPosition.z,
								g_Camera.m_vView.x,	  g_Camera.m_vView.y,     g_Camera.m_vView.z,
								g_Camera.m_vUpVector.x, g_Camera.m_vUpVector.y, g_Camera.m_vUpVector.z); 
			//ModelView::lookAt(g_modelView, x, y, z, x + lx, y + ly, z + lz, 0.0f, 1.0f, 0.0f);

		// ... to get the final model view matrix
		Matrix::multMatrix(g_modelView, g_modelView, model);
		glUniformMatrix4fv(g_modelViewLocation, 1, GL_FALSE, g_modelView);

		glDrawElements(GL_TRIANGLES, CLoad3DS::models3D.at(i).totalFaces*3, GL_UNSIGNED_INT, 0);
		}
	//************************************************************************************************
	//bis hier wird alles in den 1. frame buffer gerendert
	//************************************************************************************************
	if(bloom)
	{
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glBindFramebuffer(GL_FRAMEBUFFER, fbo2);
		glViewport(0,0,wWidth,wHeight);
		//printf("\nw: %d - h: %d", wWidth, wHeight);
		glClearColor(0.0, 0.0, 0.0, 1.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer

		//das in den frame buffer gerenderte bild wird geladen und auf ein quad gemapped
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, img);

		glUseProgram(bloomProg);
		glUniform1i(glGetUniformLocation(bloomProg, "renderpass"), 1);

		glBindBuffer(GL_ARRAY_BUFFER, g_vertices);
		glVertexAttribPointer(bl_vertexLocation, 4, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_vertexLocation);

		glBindBuffer(GL_ARRAY_BUFFER, g_normals);
		glVertexAttribPointer(bl_normalLocation, 3, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_normalLocation);

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indices);

		glBindBuffer(GL_ARRAY_BUFFER, g_texCoords);
		glVertexAttribPointer(bl_texCoordLocation, 2, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_texCoordLocation);
		glUniform1i(bl_textureLocation, 0); //bergibt die texturlocation (0) an den shader

		glUniform1f(bl_brightnessLoc, brightness);
		glUniform1f(bl_ddLoc, dd);

		//Projection::orthoFrustum(bl_projection, wWidth, wHeight, 0, 400);
		Projection::orthoFrustum(bl_projection, wWidth, wHeight, -10000, 40000);
		//Projection::perspective(bl_projection, 40.0f, ratio, 1.0f, farplane);
		glUniformMatrix4fv(bl_projectionLocation, 1, GL_FALSE, bl_projection);

		Matrix::loadIdentity(model);
		//Matrix::translateMatrix(model,-0.319/**scale*/, -0.239/**scale*/, 4.0);
		//Matrix::translateMatrix(model, wWidth*(-0.5), wHeight*(-0.5), 4.0);
		ModelView::lookAt(bl_modelView, 0, 0, 0, 0, 0, -1, 0.0f, 1.0f, 0.0f);
		Matrix::multMatrix(bl_modelView, bl_modelView, model);
		glUniformMatrix4fv(bl_modelViewLocation, 1, GL_FALSE, bl_modelView);

		glDrawElements(GL_TRIANGLES, 2*3, GL_UNSIGNED_INT, 0);

		//************************************************************************************************
		//bis hier wird alles in den 2. frame buffer gerendered
		//************************************************************************************************

		glBindFramebuffer(GL_FRAMEBUFFER, 0);

		glClearColor(0.0, 0.0, 0.0, 1.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer

		//das in den frame buffer gerenderte bild wird geladen und auf ein quad gemapped
		//img2 ist das gebloomte bild, img ist das original
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, img2);
		glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, img);
		//printf("\n img: %d", img);
		//printf("\n img2: %d", img2);

		glUseProgram(bloomProg);
		glUniform1i(glGetUniformLocation(bloomProg, "renderpass"), 2);

		glBindBuffer(GL_ARRAY_BUFFER, g_vertices);
		glVertexAttribPointer(bl_vertexLocation, 4, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_vertexLocation);

		glBindBuffer(GL_ARRAY_BUFFER, g_normals);
		glVertexAttribPointer(bl_normalLocation, 3, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_normalLocation);

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indices);

		glBindBuffer(GL_ARRAY_BUFFER, g_texCoords);
		glVertexAttribPointer(bl_texCoordLocation, 2, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_texCoordLocation);
		glUniform1i(bl_textureLocation, 0); //bergibt die texturlocation (0) an den shader
		glUniform1i(bl_textureLocation2, 1); //bergibt die 2. textur an den shader


		Projection::orthoFrustum(bl_projection, wWidth, wHeight, -10000, 40000);
		//Projection::perspective(bl_projection, 40.0f, ratio, 1.0f, farplane);
		glUniformMatrix4fv(bl_projectionLocation, 1, GL_FALSE, bl_projection);

		Matrix::loadIdentity(model);
		//Matrix::translateMatrix(model,-0.319/**scale*/, -0.23/*9*scale*/, 4.0);
		//Matrix::translateMatrix(model, wWidth*(-0.5), wHeight*(-0.5), 4.0);
		//Matrix::translateMatrix(model, -10, 0, -111.0);
		ModelView::lookAt(bl_modelView, 0, 0, 0, 0, 0, -1, 0.0f, 1.0f, 0.0f);
		Matrix::multMatrix(bl_modelView, bl_modelView, model);
		glUniformMatrix4fv(bl_modelViewLocation, 1, GL_FALSE, bl_modelView);
	}
		glDrawElements(GL_TRIANGLES, 2*3, GL_UNSIGNED_INT, 0);
	}
	if(time.getElapsedTime() > event+16)
	{
		bloom = true;
		brightness += 0.01;
		if(brightness > 2.0)
		{	
			FSOUND_Stream_Stop (soundHandle);
			FSOUND_Stream_Close(soundHandle);
			FSOUND_Close ();
			exit(0);
		}
	}
}
else if(!level2)
{
	bloom = true;

	// First we bind the FBO so we can render to it
	if(bloom)
	{
		glBindFramebuffer(GL_FRAMEBUFFER, fbo);
		glViewport(0,0,wWidth,wHeight);
	}
	glClearColor(1.0, 1.0, 1.0, 1.0);
	//glClearColor(0.0, 0.0, 0.0, 1.0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer

	// alle models durchgehen, binden, rendern
	for(int i=0; i<CLoad3DS::models3D.size(); i++)
	{
if(CLoad3DS::models3D.at(i).levelID ==1)
{
		if (CLoad3DS::models3D.at(i).shaderID == 0)
		{
			glUseProgram(p);
			g_projectionLocation = glGetUniformLocation(p, "projectionMatrix");
			g_modelViewLocation = glGetUniformLocation(p, "modelViewMatrix");
			g_normalLocation = glGetAttribLocation(p, "normal");
			g_vertexLocation = glGetAttribLocation(p, "vertex");
			glBindFragDataLocation(p, 0, "fragColor");
			g_textureLocation = glGetUniformLocation(p, "firstTexture");
			g_texCoordLocation = glGetAttribLocation(p, "texCoord");
			g_tangentLoc = glGetAttribLocation(p, "tangent");
			g_normalMapLocation = glGetUniformLocation(p, "normalMap");
			g_lightDirLocation = glGetUniformLocation(p, "lightDir");
		}
		else if (CLoad3DS::models3D.at(i).shaderID == 1)
		{
			glUseProgram(normMapProg);
			g_projectionLocation = glGetUniformLocation(normMapProg, "projectionMatrix");
			g_modelViewLocation = glGetUniformLocation(normMapProg, "modelViewMatrix");
			g_normalLocation = glGetAttribLocation(normMapProg, "normal");
			g_vertexLocation = glGetAttribLocation(normMapProg, "vertex");
			glBindFragDataLocation(normMapProg, 0, "fragColor");
			g_textureLocation = glGetUniformLocation(normMapProg, "firstTexture");
			g_texCoordLocation = glGetAttribLocation(normMapProg, "texCoord");
			g_tangentLoc = glGetAttribLocation(normMapProg, "tangent");
			g_normalMapLocation = glGetUniformLocation(normMapProg, "normalMap");
			g_lightDirLocation = glGetUniformLocation(normMapProg, "lightDir");
		}
		else if (CLoad3DS::models3D.at(i).shaderID == 2)
		{
			glUseProgram(neonProg);
			g_projectionLocation = glGetUniformLocation(neonProg, "projectionMatrix");
			g_modelViewLocation = glGetUniformLocation(neonProg, "modelViewMatrix");
			g_normalLocation = glGetAttribLocation(neonProg, "normal");
			g_vertexLocation = glGetAttribLocation(neonProg, "vertex");
			glBindFragDataLocation(neonProg, 0, "fragColor");
			g_textureLocation = glGetUniformLocation(neonProg, "firstTexture");
			g_texCoordLocation = glGetAttribLocation(neonProg, "texCoord");
			g_tangentLoc = glGetAttribLocation(neonProg, "tangent");
			g_normalMapLocation = glGetUniformLocation(neonProg, "normalMap");
			g_lightDirLocation = glGetUniformLocation(neonProg, "lightDir");
		}
		if (CLoad3DS::models3D.at(i).shaderID == 3)
		{
			glUseProgram(flatProg);
			g_projectionLocation = glGetUniformLocation(flatProg, "projectionMatrix");
			g_modelViewLocation = glGetUniformLocation(flatProg, "modelViewMatrix");
			g_normalLocation = glGetAttribLocation(flatProg, "normal");
			g_vertexLocation = glGetAttribLocation(flatProg, "vertex");
			glBindFragDataLocation(flatProg, 0, "fragColor");
			g_textureLocation = glGetUniformLocation(flatProg, "firstTexture");
			g_texCoordLocation = glGetAttribLocation(flatProg, "texCoord");
			g_tangentLoc = glGetAttribLocation(flatProg, "tangent");
			g_normalMapLocation = glGetUniformLocation(flatProg, "normalMap");
			g_lightDirLocation = glGetUniformLocation(flatProg, "lightDir");
		}
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, /*tg_texture*/CLoad3DS::models3D.at(i).texID);

			glActiveTexture(GL_TEXTURE1);
			glBindTexture(GL_TEXTURE_2D, CLoad3DS::models3D.at(i).bumpID/*tg_normalMap*/);

			glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).vert_IND);
			glVertexAttribPointer(g_vertexLocation, 4, GL_FLOAT, GL_FALSE, 0, 0);
			glEnableVertexAttribArray(g_vertexLocation);

			glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).norm_IND);
			glVertexAttribPointer(g_normalLocation, 3, GL_FLOAT, GL_FALSE, 0, 0);
			glEnableVertexAttribArray(g_normalLocation);

			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, CLoad3DS::models3D.at(i).indi_IND);

			glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).tex_IND);
			glVertexAttribPointer(g_texCoordLocation, 2, GL_FLOAT, GL_FALSE, 0, 0);
			glEnableVertexAttribArray(g_texCoordLocation);
			glUniform1i(g_textureLocation, 0); //bergibt die texturlocation (0) an den shader
			glUniform1i(g_normalMapLocation, 1);

			glBindBuffer(GL_ARRAY_BUFFER, CLoad3DS::models3D.at(i).tang_IND);
			glVertexAttribPointer(g_tangentLoc, 4, GL_FLOAT, GL_FALSE, 0, 0);
			glEnableVertexAttribArray(g_tangentLoc);
		
		if (tcount > 1.0f)
		{
			tcount = 0.0f;
			pcounter2++;
			if(pcounter2<pointamount2-3)
				getNextPoint2();
		}

		px = 0.5 *((2 * currPoints[1][0]) + (-currPoints[0][0] + currPoints[2][0]) * tcount + (2*currPoints[0][0] - 5*currPoints[1][0] + 4*currPoints[2][0] - currPoints[3][0]) * tcount*tcount + (-currPoints[0][0] + 3*currPoints[1][0]- 3*currPoints[2][0] + currPoints[3][0]) * tcount*tcount*tcount);
		py = 0.5 *((2 * currPoints[1][1]) + (-currPoints[0][1] + currPoints[2][1]) * tcount + (2*currPoints[0][1] - 5*currPoints[1][1] + 4*currPoints[2][1] - currPoints[3][1]) * tcount*tcount + (-currPoints[0][1] + 3*currPoints[1][1]- 3*currPoints[2][1] + currPoints[3][1]) * tcount*tcount*tcount);
		pz = 0.5 *((2 * currPoints[1][2]) + (-currPoints[0][2] + currPoints[2][2]) * tcount + (2*currPoints[0][2] - 5*currPoints[1][2] + 4*currPoints[2][2] - currPoints[3][2]) * tcount*tcount + (-currPoints[0][2] + 3*currPoints[1][2]- 3*currPoints[2][2] + currPoints[3][2]) * tcount*tcount*tcount);

		vx = 0.5 *((2 * currView[1][0]) + (-currView[0][0] + currView[2][0]) * tcount + (2*currView[0][0] - 5*currView[1][0] + 4*currView[2][0] - currView[3][0]) * tcount*tcount + (-currView[0][0] + 3*currView[1][0]- 3*currView[2][0] + currView[3][0]) * tcount*tcount*tcount);
		vy = 0.5 *((2 * currView[1][1]) + (-currView[0][1] + currView[2][1]) * tcount + (2*currView[0][1] - 5*currView[1][1] + 4*currView[2][1] - currView[3][1]) * tcount*tcount + (-currView[0][1] + 3*currView[1][1]- 3*currView[2][1] + currView[3][1]) * tcount*tcount*tcount);
		vz = 0.5 *((2 * currView[1][2]) + (-currView[0][2] + currView[2][2]) * tcount + (2*currView[0][2] - 5*currView[1][2] + 4*currView[2][2] - currView[3][2]) * tcount*tcount + (-currView[0][2] + 3*currView[1][2]- 3*currView[2][2] + currView[3][2]) * tcount*tcount*tcount);

		for(int j=0; j<3; j++)
		{
			pos[j] = CLoad3DS::models3D.at(i).position[j];
			rot[j] = CLoad3DS::models3D.at(i).rotation[j];
		}

		//Projection::orthoFrustum(g_projection, 100, 100, -1000, 400);
		Projection::perspective(g_projection, 40.0f, ratio, 1.0f, farplane);
		glUniformMatrix4fv(g_projectionLocation, 1, GL_FALSE, g_projection);

		if(cameraMode)
			skybox.translate(px, py-80, pz);
		else skybox.translate(g_Camera.m_vPosition.x, g_Camera.m_vPosition.y-80, g_Camera.m_vPosition.z);

		Matrix::loadIdentity(model);
		Matrix::translateMatrix(model, pos[0], pos[1], pos[2]);
		Matrix::rotateRzRyRxMatrixf(model, rot[0], rot[1], rot[2]);

		// ... and the view matrix ...
		if(cameraMode)
			ModelView::lookAt(g_modelView, px, py, pz, vx, vy, vz, 0.0f, 1.0f, 0.0f);
		else ModelView::lookAt(g_modelView, g_Camera.m_vPosition.x, g_Camera.m_vPosition.y, g_Camera.m_vPosition.z,
								g_Camera.m_vView.x,	  g_Camera.m_vView.y,     g_Camera.m_vView.z,
								g_Camera.m_vUpVector.x, g_Camera.m_vUpVector.y, g_Camera.m_vUpVector.z); 
			//ModelView::lookAt(g_modelView, x, y, z, x + lx, y + ly, z + lz, 0.0f, 1.0f, 0.0f);

		// ... to get the final model view matrix
		Matrix::multMatrix(g_modelView, g_modelView, model);
		glUniformMatrix4fv(g_modelViewLocation, 1, GL_FALSE, g_modelView);
}
		glDrawElements(GL_TRIANGLES, CLoad3DS::models3D.at(i).totalFaces*3, GL_UNSIGNED_INT, 0);
		}
	//************************************************************************************************
	//bis hier wird alles in den 1. frame buffer gerendert
	//************************************************************************************************
	if(bloom)
	{
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glBindFramebuffer(GL_FRAMEBUFFER, fbo2);
		glViewport(0,0,wWidth,wHeight);
		//printf("\nw: %d - h: %d", wWidth, wHeight);
		glClearColor(0.0, 0.0, 0.0, 1.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer

		//das in den frame buffer gerenderte bild wird geladen und auf ein quad gemapped
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, img);

		glUseProgram(bloomProg);
		glUniform1i(glGetUniformLocation(bloomProg, "renderpass"), 1);

		glBindBuffer(GL_ARRAY_BUFFER, g_vertices);
		glVertexAttribPointer(bl_vertexLocation, 4, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_vertexLocation);

		glBindBuffer(GL_ARRAY_BUFFER, g_normals);
		glVertexAttribPointer(bl_normalLocation, 3, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_normalLocation);

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indices);

		glBindBuffer(GL_ARRAY_BUFFER, g_texCoords);
		glVertexAttribPointer(bl_texCoordLocation, 2, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_texCoordLocation);
		glUniform1i(bl_textureLocation, 0); //bergibt die texturlocation (0) an den shader

		glUniform1f(bl_brightnessLoc, brightness);
		glUniform1f(bl_ddLoc, dd);

		//Projection::orthoFrustum(bl_projection, wWidth, wHeight, 0, 400);
		Projection::orthoFrustum(bl_projection, wWidth, wHeight, -10000, 40000);
		//Projection::perspective(bl_projection, 40.0f, ratio, 1.0f, farplane);
		glUniformMatrix4fv(bl_projectionLocation, 1, GL_FALSE, bl_projection);

		Matrix::loadIdentity(model);
		//Matrix::translateMatrix(model,-0.319/**scale*/, -0.239/**scale*/, 4.0);
		//Matrix::translateMatrix(model, wWidth*(-0.5), wHeight*(-0.5), 4.0);
		ModelView::lookAt(bl_modelView, 0, 0, 0, 0, 0, -1, 0.0f, 1.0f, 0.0f);
		Matrix::multMatrix(bl_modelView, bl_modelView, model);
		glUniformMatrix4fv(bl_modelViewLocation, 1, GL_FALSE, bl_modelView);

		glDrawElements(GL_TRIANGLES, 2*3, GL_UNSIGNED_INT, 0);

		//************************************************************************************************
		//bis hier wird alles in den 2. frame buffer gerendered
		//************************************************************************************************

		glBindFramebuffer(GL_FRAMEBUFFER, 0);

		glClearColor(0.0, 0.0, 0.0, 1.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer

		//das in den frame buffer gerenderte bild wird geladen und auf ein quad gemapped
		//img2 ist das gebloomte bild, img ist das original
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, img2);
		glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, img);
		//printf("\n img: %d", img);
		//printf("\n img2: %d", img2);

		glUseProgram(bloomProg);
		glUniform1i(glGetUniformLocation(bloomProg, "renderpass"), 2);

		glBindBuffer(GL_ARRAY_BUFFER, g_vertices);
		glVertexAttribPointer(bl_vertexLocation, 4, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_vertexLocation);

		glBindBuffer(GL_ARRAY_BUFFER, g_normals);
		glVertexAttribPointer(bl_normalLocation, 3, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_normalLocation);

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indices);

		glBindBuffer(GL_ARRAY_BUFFER, g_texCoords);
		glVertexAttribPointer(bl_texCoordLocation, 2, GL_FLOAT, GL_FALSE, 0, 0);
		glEnableVertexAttribArray(bl_texCoordLocation);
		glUniform1i(bl_textureLocation, 0); //bergibt die texturlocation (0) an den shader
		glUniform1i(bl_textureLocation2, 1); //bergibt die 2. textur an den shader


		Projection::orthoFrustum(bl_projection, wWidth, wHeight, -10000, 40000);
		//Projection::perspective(bl_projection, 40.0f, ratio, 1.0f, farplane);
		glUniformMatrix4fv(bl_projectionLocation, 1, GL_FALSE, bl_projection);

		Matrix::loadIdentity(model);
		//Matrix::translateMatrix(model,-0.319/**scale*/, -0.23/*9*scale*/, 4.0);
		//Matrix::translateMatrix(model, wWidth*(-0.5), wHeight*(-0.5), 4.0);
		//Matrix::translateMatrix(model, -10, 0, -111.0);
		ModelView::lookAt(bl_modelView, 0, 0, 0, 0, 0, -1, 0.0f, 1.0f, 0.0f);
		Matrix::multMatrix(bl_modelView, bl_modelView, model);
		glUniformMatrix4fv(bl_modelViewLocation, 1, GL_FALSE, bl_modelView);

		glDrawElements(GL_TRIANGLES, 2*3, GL_UNSIGNED_INT, 0);
	}
	if(time.getElapsedTime() > event_bloom03)
	{
		brightness += 0.01;
		if(brightness > 2.0)
			switchLevel();
	}
}
	glutSwapBuffers();
	angle += 0.2*fps.speedfactor;
	//angle = 0;
	tcount += 0.01f*fps.speedfactor;
	lightpath += 0.1f;
	//dd += 0.00005;
	//dd = 0.005;
	//brightness += 0.001;
	//brightness = 1.0;
}

// Get the horizontal and vertical screen sizes in pixel
void GetDesktopResolution(int& horizontal, int& vertical)
{
	RECT desktop;
	// Get a handle to the desktop window
	const HWND hDesktop = GetDesktopWindow();
	// Get the size of screen to the variable desktop
	GetWindowRect(hDesktop, &desktop);
	// The top left corner will have coordinates (0,0)
	// and the bottom right corner will have coordinates
	// (horizontal, vertical)
	horizontal = desktop.right;
	vertical = desktop.bottom;
}

void setupFBO()
{
	// Setup our FBO
	glGenFramebuffers(1, &fbo);
	glBindFramebuffer(GL_FRAMEBUFFER, fbo);
	// Create the render buffer for depth	
	glGenRenderbuffers(1, &depthBuffer);
	glBindRenderbuffer(GL_RENDERBUFFER, depthBuffer);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, wWidth, wHeight);
	// Now setup a texture to render to
	glGenTextures(1, &img);
	glBindTexture(GL_TEXTURE_2D, img);
	printf("\n******************************fbo tex: %d", img);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8,  wWidth, wHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	//  The following 3 lines enable mipmap filtering and generate the mipmap data so rendering works
	//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	//	glGenerateMipmapEXT(GL_TEXTURE_2D);

	// And attach it to the FBO so we can render to it
	glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, img, 0);
	// Attach the depth render buffer to the FBO as it's depth attachment
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depthBuffer);

	// Setup our FBO
	glGenFramebuffers(1, &fbo2);
	glBindFramebuffer(GL_FRAMEBUFFER, fbo2);
	// Create the render buffer for depth	
	glGenRenderbuffers(1, &depthBuffer2);
	glBindRenderbuffer(GL_RENDERBUFFER, depthBuffer2);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, wWidth, wHeight);
	// Now setup a texture to render to
	glGenTextures(1, &img2);
	glBindTexture(GL_TEXTURE_2D, img2);
	printf("\n******************************fbo tex: %d", img2);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8,  wWidth, wHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	//  The following 3 lines enable mipmap filtering and generate the mipmap data so rendering works
	//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	//	glGenerateMipmapEXT(GL_TEXTURE_2D);
	//printf("\nwidth: %d - height: %d", 	GLUT_WINDOW_WIDTH, 	GLUT_WINDOW_HEIGHT);

	// And attach it to the FBO so we can render to it
	glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, img2, 0);
	// Attach the depth render buffer to the FBO as it's depth attachment
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depthBuffer2);

	GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER);
	if(status != GL_FRAMEBUFFER_COMPLETE)
		exit(0);

	glBindFramebuffer(GL_FRAMEBUFFER, 0);	// Unbind the FBO for now
}

void main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowPosition(150, 150);
	glutInitWindowSize(wWidth, wHeight);
	glutCreateWindow("elevator suite");
	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);
	glutReshapeFunc(resize);
	glutIgnoreKeyRepeat(1);
	glutKeyboardFunc(processKeys);
	glutSpecialFunc(processSpecialKeys);
	glutSpecialUpFunc(releaseSpecialKeys);
	glutPassiveMotionFunc(passiveMouseMotion);
	glutSetCursor(GLUT_CURSOR_NONE);

	if(fullscreen)
	{
		GetDesktopResolution(wWidth, wHeight);
		printf("\n**********************************w: %d - h: %d", wWidth, wHeight);
	}
	g_Camera.SetScreenRes(wWidth, wHeight);
	//g_Camera.PositionCamera(0.0, 1.5f, 6.0, 0.0, 1.5f, 5.0, 0.0, 1.0, 0.0);
	g_Camera.PositionCamera(0.0, 0.0f, 6.0, 0.0, 1.5f, 5.0, 0.0, 1.0, 0.0);
	printf("\n2222 w: %d - h: %d", wWidth, wHeight);

	initGL();
	glewExperimental = GL_TRUE;
	glewInit();
	fps.Init(60.0);

	skybox = CLoad3DS("models/skybox.3ds", 3, 1);
	skybox2 = CLoad3DS("models/skybox2.3ds", 3, 2);
	skybox2.translate(0,0,1000);

	port = CLoad3DS("models/port.3ds",1, 2);
	ufo = CLoad3DS("models/ufo.3ds",1, 2);
	platform = CLoad3DS("models/platform.3ds",1, 2);
	lvl1 = CLoad3DS("models/room.3ds",1, 1);
	ufo.translate(0, 0, 1000);
	port.translate(0, 0, 1000);
	platform.translate(0, 0, 1000);

	setShaders();
	//printf("\nvec%d", testvec.size());

	if (!glewIsSupported("GL_VERSION_3_0"))
	{
		printf("OpenGL 3.0 not supported.");
	}
	
	printf("\nmodel count: %d\n", CLoad3DS::models3D.size());

	//loadXML();
	cameraMode = true;
	TiXmlDocument temp("campath.xml");
	bool loaded = temp.LoadFile();

	if(loaded)
	{
		printf("\nxml file loaded.");
	}
	else printf("\nxml file not found.");
	float xc, yc, zc;

	TiXmlHandle hDoc(&temp);
	TiXmlElement* pElem=hDoc.FirstChildElement().Element();
	TiXmlHandle hRoot=TiXmlHandle(pElem);

	pcounter = 0;
	hRoot.FirstChild("points").Element()->QueryIntAttribute("amount", &pointamount);
	printf("\nnumber of points: %d", pointamount);

	camNode=hRoot.FirstChild( "points" ).FirstChild().Element();

	//save the first 4 points of our camera path
	for (int i=0; i<4; i++)
	{
		camNode->QueryFloatAttribute("x", &xc); // If this fails, original value is left as-is
		camNode->QueryFloatAttribute("y", &yc);
		camNode->QueryFloatAttribute("z", &zc);
		printf("\npos values x='%f', y='%f', z='%f'", xc, yc, zc);
		currentPoints[i][0] = xc;
		currentPoints[i][1] = yc;
		currentPoints[i][2] = zc;

		camNode=camNode->NextSiblingElement();
		camNode->QueryFloatAttribute("xv", &xc);
		camNode->QueryFloatAttribute("yv", &yc);
		camNode->QueryFloatAttribute("zv", &zc);
		currentView[i][0] = xc;
		currentView[i][1] = yc;
		currentView[i][2] = zc;
		printf("\nview values x='%f', y='%f', z='%f'", xc, yc, zc);

		if(i!=3)
			camNode=camNode->NextSiblingElement();
	}

	//*************************************************************************
	//initiation of 2nd cam path
	TiXmlDocument temp2("campath2.xml");
	bool loaded2 = temp2.LoadFile();

	if(loaded2)
	{
		printf("\nxml file loaded.");
	}
	else printf("\nxml file not found.");
	float xc2, yc2, zc2;

	TiXmlHandle hDoc2(&temp2);
	TiXmlElement* pElem2=hDoc2.FirstChildElement().Element();
	TiXmlHandle hRoot2=TiXmlHandle(pElem2);

	pcounter2 = 0;
	hRoot2.FirstChild("points").Element()->QueryIntAttribute("amount", &pointamount2);
	printf("\nnumber of points: %d", pointamount2);

	cam2Node=hRoot2.FirstChild( "points" ).FirstChild().Element();

	//save the first 4 points of our camera path
	for (int i=0; i<4; i++)
	{
		cam2Node->QueryFloatAttribute("x", &xc2); // If this fails, original value is left as-is
		cam2Node->QueryFloatAttribute("y", &yc2);
		cam2Node->QueryFloatAttribute("z", &zc2);
		printf("\npos values x='%f', y='%f', z='%f'", xc2, yc2, zc2);
		currPoints[i][0] = xc2;
		currPoints[i][1] = yc2;
		currPoints[i][2] = zc2;

		cam2Node=cam2Node->NextSiblingElement();
		cam2Node->QueryFloatAttribute("xv", &xc2);
		cam2Node->QueryFloatAttribute("yv", &yc2);
		cam2Node->QueryFloatAttribute("zv", &zc2);
		currView[i][0] = xc2;
		currView[i][1] = yc2;
		currView[i][2] = zc2;
		printf("\nview values x='%f', y='%f', z='%f'", xc2, yc2, zc2);

		if(i!=3)
			cam2Node=cam2Node->NextSiblingElement();
	}
	
	setupFBO();

	bloom = false;
	//testmode = true;
	lightpath = -50.0;



	if(fullscreen)
		glutFullScreen();

	FSOUND_Init(44100, 32, 0);
	soundHandle = FSOUND_Stream_Open("bgmusic.mp3",0, 0, 0);
	FSOUND_Stream_Play(0,soundHandle);
	if(level2)
		switchLevel();

	time.start();
	glutMainLoop();
}
