#include "Image.h"

#include <stdio.h>

GLvoid Image::swapColorChannel(GLuint width, GLuint height, GLenum format, GLubyte* pixelData)
{
	GLubyte temp;
	GLubyte bytesPerPixel = 3;
	
	if (format == GL_RGBA)
	{
		bytesPerPixel = 4;
	}
	
	// swap the R and B values to get RGB since the bitmap color format is in BGR
	for (GLuint i = 0; i < width*height*bytesPerPixel; i+=bytesPerPixel)
	{
		temp = pixelData[i];
		pixelData[i] = pixelData[i+2];
		pixelData[i+2] = temp;
	}
}

GLboolean Image::loadTGA(const char* filename, GLushort& width, GLushort& height, GLenum& format, GLubyte** pixelData)
{
	FILE*				file;
	GLubyte				type;
	GLubyte				bitsPerPixel;				

	// check, if we have a valid pointer
	if (!filename || !pixelData)
	{
		return GL_FALSE;
	}

	// open filename in "read binary" mode
	file = fopen(filename, "rb");

 	// seek through the tga header, up to the type:
 	fseek(file, 2, SEEK_CUR);

 	// read the type
 	fread(&type, 1, 1, file);

	// check the type	
	if (type != 2 && type != 10 && type != 11)
	{
		fclose(file);
		return GL_FALSE;
	}

 	// seek through the tga header, up to the width/height:
 	fseek(file, 9, SEEK_CUR);
	
 	// read the width
 	fread(&width, 2, 1, file);
 
 	// read the height 
 	fread(&height, 2, 1, file);

	// read the bits per pixel
	fread(&bitsPerPixel, 1, 1, file);

	// check the pixel depth
	if (bitsPerPixel != 8 && bitsPerPixel != 24 && bitsPerPixel != 32)
	{
		fclose(file);
		return GL_FALSE;
	}
	else
	{	
		format = GL_ALPHA;	
		if (bitsPerPixel == 24)
		{
			format = GL_RGB;
		}
		else if (bitsPerPixel == 32)
		{
			format = GL_RGBA;
		}
	}

	// move file pointer to beginning of targa  data
	fseek(file, 1, SEEK_CUR);

	// allocate enough memory for the targa  data
	*pixelData = new GLubyte[width*height*bitsPerPixel/8];

	// verify memory allocation
	if (!pixelData)
	{
		fclose(file);
		return GL_FALSE;
	}

	if (type == 2)
	{
		// read in the raw data
		fread(*pixelData, 1, width*height*bitsPerPixel/8, file);
	}
	else
	{
		// RLE encoded
		GLuint pixelsRead = 0;

		while (pixelsRead < (GLuint)width*height)
		{
			GLubyte amount;

			fread(&amount, 1, 1, file);

			if (amount & 0x80)
			{
				amount&=0x7F;

				amount++;

				// read in the rle data
				fread(&(*pixelData)[pixelsRead*bitsPerPixel/8], 1, bitsPerPixel/8, file);

				for (GLuint i = 1; i < amount; i++)
				{
					for (GLuint k = 0; k < (GLuint)bitsPerPixel/8; k++)
					{
						(*pixelData)[(pixelsRead+i)*bitsPerPixel/8+k] = (*pixelData)[pixelsRead*bitsPerPixel/8+k];
					}
				}
			}
			else
			{
				amount&=0x7F;

				amount++;
				
				// read in the raw data
				fread(&(*pixelData)[pixelsRead*bitsPerPixel/8], 1, (GLuint)amount*bitsPerPixel/8, file);				
			}

			pixelsRead += amount;
		}
	}

	// swap the color if necessary
	if (bitsPerPixel == 24 || bitsPerPixel == 32)
	{
		swapColorChannel(width, height, format, *pixelData);
	}

	// close the file
	fclose(file);
	
	return GL_TRUE;
}
