#include "Matrix.h"

#include <math.h>
#define PIf		3.1415927410125732f

GLvoid Matrix::loadIdentity(GLfloat matrix[16])
{
	matrix[0] = 1.0f;
	matrix[1] = 0.0f;
	matrix[2] = 0.0f;
	matrix[3] = 0.0f;

	matrix[4] = 0.0f;
	matrix[5] = 1.0f;
	matrix[6] = 0.0f;
	matrix[7] = 0.0f;

	matrix[8] = 0.0f;
	matrix[9] = 0.0f;
	matrix[10] = 1.0f;
	matrix[11] = 0.0f;

	matrix[12] = 0.0f;
	matrix[13] = 0.0f;
	matrix[14] = 0.0f;
	matrix[15] = 1.0f;
}

GLvoid Matrix::translateMatrix(GLfloat result[16], GLfloat x, GLfloat y, GLfloat z)
{
	GLfloat matrix[16];

	loadIdentity(matrix);

	matrix[12] = x;
	matrix[13] = y;
	matrix[14] = z;
	
	multMatrix(result, result, matrix);
}

GLvoid Matrix::multMatrix(GLfloat result[16], const GLfloat matrix0[16], const GLfloat matrix1[16])
{
	GLuint i;

	GLfloat temp[16];

	for (i = 0; i < 16; i++)
	{
		temp[i] = 0.0f;
		
		for(GLuint k = 0; k < 4; k++)
		{
			//			  		row   column   		   row column
			temp[i] += matrix0[(i%4)+(k*4)] * matrix1[(k)+((i/4)*4)];
		}
	}
		
	for (i = 0; i < 16; i++)
	{
		result[i] = temp[i];
	}
}

GLvoid Matrix::rotateRzRyRxMatrixf(GLfloat result[16], GLfloat anglex, GLfloat angley, GLfloat anglez)
{
	GLfloat matrix[16];
	GLfloat rx = 2.0f*PIf*anglex/360.0f;
	GLfloat ry = 2.0f*PIf*angley/360.0f;
	GLfloat rz = 2.0f*PIf*anglez/360.0f;
	GLfloat sx = sinf(rx);
	GLfloat cx = cosf(rx);
	GLfloat sy = sinf(ry);
	GLfloat cy = cosf(ry);
	GLfloat sz = sinf(rz);
	GLfloat cz = cosf(rz);
	
	loadIdentity(matrix);

	matrix[0] = cy*cz;
	matrix[1] = cy*sz;
	matrix[2] = -sy;

	matrix[4] = cz*sx*sy-cx*sz;
	matrix[5] = cx*cz+sx*sy*sz;
	matrix[6] = cy*sx;

	matrix[8] = cx*cz*sy+sx*sz;
	matrix[9] = -cz*sx+cx*sy*sz;
	matrix[10] = cx*cy;

	multMatrix(result, result, matrix);
}
