/**
 * OpenGL 3.1 with GLEW and GLUT
 *
 * @author	Norbert Nopper norbert@nopper.tv
 * @version	1.0
 *
 * Homepage: http://nopper.tv
 *
 * Copyright Norbert Nopper
 */

#include "Projection.h"

#include <math.h>
#define PIf		3.1415927410125732f

GLvoid Projection::perspective(GLfloat result[16], GLfloat fovy, GLfloat aspect, GLfloat zNear, GLfloat zFar)
{
    GLfloat xmin, xmax, ymin, ymax;

	ymax = zNear * tanf(fovy * PIf / 360.0f);
    ymin = -ymax;
    xmin = ymin * aspect;
    xmax = ymax * aspect;

    frustum(result, xmin, xmax, ymin, ymax, zNear, zFar);
}

//GLvoid Projection::ortho(GLfloat result[16], GLfloat fovy, GLfloat aspect, GLfloat zNear, GLfloat zFar)
//{
//	GLfloat xmin, xmax, ymin, ymax;
//
//	ymax = zNear * tanf(fovy * PIf / 360.0f);
//	ymin = -ymax;
//	xmin = ymin * aspect;
//	xmax = ymax * aspect;
//
//	//orthoFrustum(result, xmin, xmax, ymin, ymax, zNear, zFar);
//}
GLvoid Projection::orthoFrustum(GLfloat result[16], GLfloat r, GLfloat b, GLfloat nearVal, GLfloat farVal)
{
	result[0] = 2.0f/r;
	result[1] = 0.0f;
	result[2] = 0.0f;
	result[3] = 0.0f;
	result[4] = 0.0f;
	result[5] = 2.0f/-b;
	result[6] = 0.0f;
	result[7] = 0.0f;
	result[8] = 0.0f;
	result[9] = 0.0f;
	result[10] = -2/(farVal-nearVal);
	result[11] = 0.0f;
	result[12] = -((r+0)/(r-0));
	result[13] = -((0+b)/(0-b));
	result[14] = -((farVal+nearVal)/(farVal-nearVal));
	result[15] = 1.0f;
}

//GLvoid Projection::orthoFrustum(GLfloat result[16], GLfloat left, GLfloat right, GLfloat bottom, GLfloat top, GLfloat nearVal, GLfloat farVal)
//{
//	result[0] = 2.0f/(right-left);
//	result[1] = 0.0f;
//	result[2] = 0.0f;
//	result[3] = 0.0f;
//	result[4] = 0.0f;
//	result[5] = 2.0f/(top-bottom);
//	result[6] = 0.0f;
//	result[7] = 0.0f;
//	result[8] = 0.0f;
//	result[9] = 0.0f;
//	result[10] = 1/(farVal-nearVal);
//	result[11] = 0.0f;
//	result[12] = 0.0f;
//	result[13] = 0.0f;
//	result[14] = -nearVal/(farVal-nearVal);
//	result[15] = 0.0f;
//}

GLvoid Projection::frustum(GLfloat result[16], GLfloat left, GLfloat right, GLfloat bottom, GLfloat top, GLfloat nearVal, GLfloat farVal)
{
	result[0] = 2.0f*nearVal/(right-left);
	result[1] = 0.0f;
	result[2] = 0.0f;
	result[3] = 0.0f;
	result[4] = 0.0f;
	result[5] = 2.0f*nearVal/(top-bottom);
	result[6] = 0.0f;
	result[7] = 0.0f;
	result[8] = (right+left)/(right-left);
	result[9] = (top+bottom)/(top-bottom);
	result[10] = -(farVal+nearVal)/(farVal-nearVal);
	result[11] = -1.0f;
	result[12] = 0.0f;
	result[13] = 0.0f;
	result[14] = -(2.0f*farVal*nearVal)/(farVal-nearVal);
	result[15] = 0.0f;
}