/**
 * OpenGL 3.1 with GLEW and GLUT
 *
 * @author	Norbert Nopper norbert@nopper.tv
 * @version	1.0
 *
 * Homepage: http://nopper.tv
 *
 * Copyright Norbert Nopper
 */

#pragma once

#include <GL/glew.h>

/**
 * Class for compiling and linking a shader.
 */
class ShaderProgram
{

private:

	/**
	 * Vertex shader.
	 */
	GLuint m_vertex;

	/**
	 * Fragment shader.
	 */
	GLuint m_fragment;

	/**
	 * Shader program.
	 */
	GLuint m_program;

public:

	/**
	 * Constructor
	 */
	ShaderProgram(void);

	/**
	 * Destructor
	 */
	~ShaderProgram(void);

	/**
	 * Builds the given shader.
	 *
	 * @param vertexSource the source of a vertex shader
	 * @param fragmentSource the source of a frgament shader
	 */
	GLboolean build(const char** vertexSource, const char** fragmentSource);

	/**
	 * Deletes all shaders and the program.
	 */
	void deleteAll(void);

	/**
	 * Returns the created program.
	 *
	 * @return the program
	 */
	GLuint getProgram(void);

};
